float4 main() : SV_TARGET
{
	if (!(0) && (0) || (!1))
	    return 0.0.xxxx; 
	if (0)
	    return 0.0.xxxx; 
	if (!(bool)0)
		return 0.0.xxxx;
	if (!0)
		return 0.0.xxxx;
	if (!(bool)1)
		return 0.0.xxxx;
	if (!1)
		return 0.0.xxxx;
    if (0 || 1)
		return 0.0.xxxx;
    if (1 && 0)
		return 0.0.xxxx;
    if (1 || false)
		return 0.0.xxxx;
    if (true && 1)
		return 0.0.xxxx;
}