string s = "string1";
string e = "";
string bracket < string a = "nested" ; > ; 
string brackets < string b = "nest1" ; string c = "nest2" ; float test [ 4 ] = { 1.0 , 1.0 , 1.0 , 1.0 } ; vector<float, 3> a = float3(2.0); > ;
string brackete1 <  > ;
string brackete2 < ; > ;
string brackete3 < ; ; > ;

float main(float f)
{
    return f;
}